@@ -1,22 +1,8 @@
 /*
- * <one line to give the library's name and an idea of what it does.>
- * Copyright (C) 2014  Vishesh Handa <me@vhanda.in>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
- */
+    SPDX-FileCopyrightText: 2014 Vishesh Handa <me@vhanda.in>
+
+    SPDX-License-Identifier: LGPL-2.1-or-later
+*/
 
 #include "typeinfo.h"
 
@@ -24,7 +10,8 @@
 
 using namespace KFileMetaData;
 
-class TypeInfo::Private {
+class KFileMetaData::TypeInfoPrivate
+{
 public:
     Type::Type type;
     QString name;
@@ -32,7 +19,7 @@ public:
 };
 
 TypeInfo::TypeInfo(Type::Type type)
-    : d(new Private)
+    : d(new TypeInfoPrivate)
 {
     d->type = type;
 
@@ -90,14 +77,11 @@ TypeInfo::TypeInfo(Type::Type type)
 }
 
 TypeInfo::TypeInfo(const TypeInfo& ti)
-    : d(new Private(*ti.d))
+    : d(new TypeInfoPrivate(*ti.d))
 {
 }
 
-TypeInfo::~TypeInfo()
-{
-    delete d;
-}
+TypeInfo::~TypeInfo() = default;
 
 TypeInfo& TypeInfo::operator=(const TypeInfo& rhs)
 {
@@ -105,9 +89,9 @@ TypeInfo& TypeInfo::operator=(const TypeInfo& rhs)
     return *this;
 }
 
-bool TypeInfo::operator==(const TypeInfo& rhs)
+bool TypeInfo::operator==(const TypeInfo& rhs) const
 {
-    return d->type == rhs.d->type && d->name == rhs.d->name && d->displayName == rhs.d->displayName;
+    return std::tie(d->type, d->name, d->displayName) == std::tie(rhs.d->type, rhs.d->name, rhs.d->displayName);
 }
 
 QString TypeInfo::displayName() const
@@ -127,7 +111,7 @@ Type::Type TypeInfo::type() const
 
 TypeInfo TypeInfo::fromName(const QString& name)
 {
-    for (int t = static_cast<int>(Type::FirstType); t != static_cast<int>(Type::LastType); t++) {
+    for (int t = static_cast<int>(Type::FirstType); t <= static_cast<int>(Type::LastType); t++) {
         TypeInfo ti(static_cast<Type::Type>(t));
         if (ti.name().compare(name, Qt::CaseInsensitive) == 0) {
             return ti;
